module IRC
  module Handlers
    class MembershipHandler < Event::Handler
      def on_connection_established
        @members = { }
      end
      
      def on_server_supports
        if memo[:feature] == "PREFIX"
          @prefixes = memo[:value][/\(.*?\)(.*)/, 1]
        end
      end
      
      def on_join
        add_membership(response.text, response.from.nickname)
      end
      
      def on_353 # RPL_NAMREPLY
        response.text.split.each do |nick|
          nick = nick.sub(/^[#{Regexp.escape(@prefixes)}]+/, "")
          add_membership(response.args[2], nick)
        end
      end
      
      def on_nick
        old_nickname, new_nickname = response.from.nickname, response.text
        channels_for(old_nickname).each do |channel|
          remove_membership(channel, old_nickname)
          add_membership(channel, new_nickname)
        end
      end

      def on_part
        remove_membership(response.args[0], response.from.nickname)
      end
      
      def on_kick
        remove_membership(response.args[0], response.args[1])
      end
      
      def on_quit
        nickname = response.from.nickname
        channels_for(nickname).each do |channel|
          remove_membership(channel, nickname)
        end
      end
      
      def channel
        response.args.first[/^#.*/]
      end

      def channels
        members.keys
      end
      
      def nicknames(channel)
        (members[normalize(channel)] || []).dup
      end
      
      protected
        attr_reader :members
        
        def member_of?(channel, nickname)
          (members[normalize(channel)] || []).any? do |member|
            normalize(member) == normalize(nickname)
          end
        end
        
        def channels_for(nickname)
          channels.select do |channel|
            member_of?(channel, nickname)
          end
        end
        
        def add_membership(channel, nickname)
          unless member_of?(channel, nickname)
            (members[normalize(channel)] ||= []) << nickname
          end
        end
        
        def remove_membership(channel, nickname)
          if normalize(self.nickname) == normalize(nickname)
            members.delete(normalize(channel))
          else
            (members[normalize(channel)] || []).delete_if do |member| 
              normalize(member) == normalize(nickname)
            end
          end
        end
    end
  end
end
